class Constants:

  _DOC =  "_doc"
  _INDEX =  "_index"
  _SOURCE =  "_source"
  _TYPE =  "_type"
  _WRITE =  "-$_write"
  CONFIG =  "config"
  CONTROL =  "control"
  ELASTICSEARCH =  "elasticsearch"
  ENABLED =  "enabled"
  HOST =  "host"
  INDEX =  "index"
  NAME =  "name"
  OWNER =  "owner"
  PASSWORD =  "password"
  PORT =  "port"
  PROFILE_ID =  "profile_id"
  PROTOCOL =  "protocol"
  TYPE =  "type"
  USERNAME =  "username"
  decrypt_key =  "SnappyFlow123456"
  TOKEN = 'token'

